' RotorTest -- test the math for a moving Enigma rotor
' For these tests we use the forward and reverse mapping
' for the 'I" rotor.
' Rev 1.0.0 William M Leue 12/27/2020

option default integer
option base 1

const NUM_CHARS = 26
const ALL = -1

dim rotor_mappings(2, NUM_CHARS)

' main program
cls
ReadRotorMappings
RunTests ALL
end

' read forward and reverse mappings
sub ReadRotorMappings
  local i
  for i = 1 to NUM_CHARS
    read rotor_mappings(1, i)
  next i
  for i = 1 to NUM_CHARS
    read rotor_mappings(2, i)
  next i
end sub

' perform correct wraparound into alphabet range
function wrap(k)
  local w
  if k >= 1 and k <= NUM_CHARS then
    w = k
  else if k > 26 then
    w = k mod NUM_CHARS
  else if k <= 0 then
    w = NUM_CHARS + k
  end if
  wrap = w
end function

' given the input wire, rotor position, and ring setting,
' return the rotor nest input index
function NestInput(iwire, rpos, ring)
  NestInput = wrap(iwire + (rpos-1) - (ring-1))
end function

' given a nest input index, return the nest output index,
' depending on the signal direction.
function NestOutput(inest, dir)
  if dir > 0 then
    NestOutput = rotor_mappings(1, inest)
  else
    NestOutput = rotor_mappings(2, inest)
  end if
end function

' given a nest output index, rotor position, and ring setting,
' return the output wire index.
function OutputWire(onest, rpos, ring)
  OutputWire = wrap(onest - rpos + ring)
end function

' test driver
sub RunTests first, last
  local tnum  ' test number
  local rpos  ' rotor position 1..26
  local ring  ' ring setting 1..26
  local iwire ' input wire 1..26
  local inest ' rotor nest input index 1..26
  local onest ' rotor nest output index 1..26
  local owire ' output wire 1..26
  local dir   ' direction: 1 = forward, -1 = reverse
  local i

  if first = -1 then
    first = 1
    last = 100
  end if

  if first > 1 then
    for i = 1 to first-1
      read tnum, rpos, ring, iwire, dir
    next i
  end if

  for i = first to last
    on error skip 1
    read tnum, rpos, ring, iwire, dir
    if MM.ERRNO > 0 then
      print "All tests Run"
      exit for
    end if

    inest = NestInput(iwire, rpos, ring)
    onest = NestOutput(inest, dir)
    owire = OutputWire(onest, rpos, ring)
    PrintResult tnum, iwire, rpos, ring, dir, inest, onest, owire

    z$ = INKEY$
    print "Press any Key to Continue"
    do
      z$ = INKEY$
    loop until z$ <> ""

  next i
  
end sub

' print the result of a test
sub PrintResult tnum, iwire, rpos, ring, dir, inest, onest, owire
  print ""
  print "Test Number " + str$(tnum)
  print "-------------"
  print "Input Wire  : " + chr$(65+iwire-1) + "(" + str$(iwire) + ")"
  print "Rotor Pos   : " + chr$(65+rpos-1)  + "(" + str$(rpos)  + ")"
  print "Ring Setng  : " + chr$(65+ring-1)  + "(" + str$(ring)  + ")"
  print "Direction   : " + str$(dir)
  print "------------"
  print "Nest Input  : " + chr$(65+inest-1) + "(" + str$(inest) + ")"
  print "Nest Output : " + chr$(65+onest-1) + "(" + str$(onest) + ")"
  print "Output Wire : " + chr$(65+owire-1) + "(" + str$(owire) + ")"
  print ""
end sub 


' forward and reverse mappings for the 'I' Enigma rotor
data 5,11,13,6,12,7,4,17,22,26,14,20,15,23,25,8,24,21,19,16,1,9,2,18,3,10
data 21,23,25,7,1,4,6,16,22,26,2,5,3,11,13,20,8,24,19,12,18,9,14,17,15,10

' test inputs: test number, rotor position, ring setting, input wire, direction
data 1,  1,  1, 17,  1
data 2,  7,  1, 17,  1
data 3, 21,  1,  4,  1
data 4, 12,  1,  9, -1
data 5, 25,  1, 14, -1
data 6,  1,  2,  1,  1
data 7, 25,  6,  1,  1
data 8,  9,  3,  5, -1
data 9, 17, 12, 20, -1
